import { FormEvent, useEffect, useState } from 'react'
import styles from './AuthorizeForm.module.css'
import IUserCreateDto from '../../interfaces/IUserCreateDto'
import { AppDispatch, AppState, useAppDispatch } from '../../store/store'
import { useLocation, useNavigate } from 'react-router-dom'
import { shallowEqual, useSelector } from 'react-redux'
import Preloader from '../../components/UI/Preloader/Preloader'
import { createUser, hideMessage, loginUser } from '../../store/user/user.slice'
import Alert from '@mui/material/Alert';

const AuthorizeForm: React.FunctionComponent = (): React.ReactElement => {
    const navigate = useNavigate()

    const dispatch: AppDispatch = useAppDispatch()
    const { isAuth, messageUser, loadingUser } = useSelector((state: AppState) => state.users, shallowEqual)
    const location = useLocation()
    const [userValues, setUserValues] = useState<IUserCreateDto>({
        username: '',
        password: ''
    })

    const [buttonDisabled, setButtonDisabled] = useState<boolean>(true)

    const [isLoginUser, setIsLoginUser] = useState<boolean>(true)

    const inputHandler = (e: React.ChangeEvent<HTMLInputElement>): void => {
        setUserValues(prevState => {
            return {...prevState, [e.target.name]: e.target.value}
        })
    }

    const toggleChangeHandler = () => {
        setIsLoginUser(!isLoginUser)
    }

    const submitHandler = async (e: FormEvent) => {
        e.preventDefault()
        if (isLoginUser){
            await dispatch(createUser(userValues))
        } else{
            await dispatch(loginUser(userValues))
        }
    }

    useEffect(() => {
        checkButton()
    },[userValues])

    useEffect(() => {
        if (isAuth){
            navigate(location.state?.from ? location.state.from : '/')
        }
    }, [isAuth])

    const checkButton = () => {
        if (userValues.username.trim() === '' || userValues.password.trim() === ''){
            setButtonDisabled(true)
        } else{
            setButtonDisabled(false)
        }
    }

    useEffect(() => {
        dispatch(hideMessage())
    }, [])

    return (
        <div className={styles.AuthorizeForm}>
            {
                loadingUser ? <Preloader/> : null
            }
            <form onSubmit={submitHandler}>
                {
                    messageUser.trim() !== '' ?
                    <Alert variant="filled" severity="error">
                        {messageUser}
                    </Alert> : null
                }
                <p>{isLoginUser ? 'Login user' : 'Authorize user'}</p>
                <p>Username:</p>
                <input
                    className={styles.login_input} 
                    placeholder='username...' 
                    name={'username'} 
                    autoComplete='off'
                    value={userValues.username}
                    onChange={inputHandler}
                />
                <p>Password:</p>
                <input
                    className={styles.login_input}  
                    placeholder='password...' 
                    name={'password'} 
                    type='password'
                    autoComplete='off'
                    value={userValues.password}
                    onChange={inputHandler}
                />
                <label style={{display: 'flex', alignItems: 'center'}}>
                    <input 
                        className={styles.toggleCheckbox} 
                        type="checkbox" 
                        checked={isLoginUser}
                        onChange={toggleChangeHandler} 
                    />
                    <div className={styles.toggleSwitch}></div>
                    <span className={styles.toggleLabel}>New user?</span>
                </label>
                <button
                    className={styles.login_btn}
                    disabled={buttonDisabled}
                >Authorize</button>
            </form>
        </div>
    )
}

export default AuthorizeForm
